/* Fast Interrupt handling
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Image Loader.

The Imote2 Image Loader is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

The Imote2 Image Loader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the Imote2 Image Loader. If not, see <http://www.gnu.org/licenses/>.
*/
#include "exception.h"
#include "exceptions.h"
#include "ledcycle.h"
#include <os_arm_bare_registers_def.h>
#include <stdio.h>

struct ICStatus {
	uint32_t ichp;
	uint32_t icmr, icmr2;
	uint32_t iclr, iclr2;
	uint32_t icfp, icfp2;
};

static void printFault(uint32_t lr, const uint32_t * fp, 
	const struct _ExcProcessorStatus * pstat,
	const struct ICStatus * icstat);

__attribute__((interrupt("FIQ"),weak))
void hplarmv_fiq() {
	/* gather some data: lr, fp, processor status */
	uint32_t lr = (uint32_t)__builtin_return_address(0);
	uint32_t * fp = (uint32_t*)__builtin_frame_address(0);
	struct _ExcProcessorStatus pstat;
	_excGetProcessorStatus(&pstat);
	
	struct ICStatus icstat = {
		.ichp = ICHP,
		.icmr = ICMR, .icmr2 = ICMR2,
		.iclr = ICLR, .iclr2 = ICLR2,
		.icfp = ICFP, .icfp2 = ICFP2
	};
	
	/* initialize exception handler */
	_excInit();
	
	/* begin in next line */
	_excUartOut("\n");
	
	while (1) {
		printFault(lr, fp, &pstat, &icstat);
		_excLedCycle(TRUE);
	}
}

static void printFault(uint32_t lr, const uint32_t * fp,
	const struct _ExcProcessorStatus * pstat,
	const struct ICStatus * icstat) {
	char buf[128];
	
	/* print the exception type */
	_excUartOut("Fast Interrupt Request (Not implemented!)\n");
	
	snprintf(buf, sizeof(buf), "ICHP: value: %d %s ICLR: 0x%08x 0x%08x\n",
		icstat->ichp & 0x3f,
		icstat->ichp & 0xf ? _EXC_LABEL_VALID : _EXC_LABEL_INVALID,
		icstat->iclr, icstat->iclr2);
	_excUartOut(buf);
	
	snprintf(buf, sizeof(buf), "ICMR: 0x%08x 0x%08x ICFP: 0x%08x 0x%08x\n",
		icstat->icmr, icstat->icmr2, icstat->icfp, icstat->icfp2);
	_excUartOut(buf);
	
	_excPrintProcessorStatus(pstat->oldlr, fp, pstat);
}

