/* Common Exception handling
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Image Loader.

The Imote2 Image Loader is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

The Imote2 Image Loader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the Imote2 Image Loader. If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef EXCEPTION_H
#define EXCEPTION_H

#include <inttypes.h>

struct _ExcProcessorStatus {
	/** \brief Saved Procressor Status Register */
	uint32_t spsr;
	/** \brief pretty printed name of previous mode */
	const char * spsrModeLabel;
	/** \brief Current Processor Status Register */
	uint32_t cpsr;
	/** \brief pretty printed name of current mode */
	const char * cpsrModeLabel;
	/** \brief SP of previous processor mode */
	uint32_t oldsp;
	/** \brief LR of previous processor mode */
	uint32_t oldlr;
	/** \brief Control Register */
	uint32_t creg;
	/** \brief PID Register */
	uint32_t pid;
};

extern const char
	* _EXC_LABEL_ENABLED,
	* _EXC_LABEL_DISABLED,
	* _EXC_LABEL_VALID,
	* _EXC_LABEL_INVALID;

void _excInit();

uint32_t _excGetFaultStatus();

uint32_t _excGetFaultAddress();

void _excGetProcessorStatus(struct _ExcProcessorStatus * stat);

void _excUartInit();

void _excUartOut(const char * msg);

void _excPrintProcessorStatus(uint32_t lr, const uint32_t * fp,
	const struct _ExcProcessorStatus * pstat);

#endif

